@@ -43,7 +43,6 @@ def initialize(finder):
         finder.ExcludeModule("posix")
     if sys.platform != "darwin":
         finder.ExcludeModule("Carbon")
-        finder.ExcludeModule("gestalt")
         finder.ExcludeModule("ic")
         finder.ExcludeModule("mac")
         finder.ExcludeModule("MacOS")
@@ -70,7 +69,6 @@ def initialize(finder):
     if sys.platform[:4] != "OpenVMS":
         finder.ExcludeModule("vms_lib")
     finder.ExcludeModule("new")
-    finder.ExcludeModule("Tkinter")
 
 
 def load_asyncio(finder, module):
@@ -91,14 +89,25 @@ def load_ceODBC(finder, module):
     finder.IncludeModule("decimal")
 
 
+def load_cryptography_hazmat_bindings__padding(finder, module):
+    """the cryptography module requires the _cffi_backend module (loaded implicitly)"""
+    finder.IncludeModule('_cffi_backend')
+
+
+def load__ctypes(finder, module):
+    """In Windows, the _ctypes module in Python >= 3.8 requires an additional dll
+       libffi-7.dll to be present in the build directory."""
+    if sys.platform == "win32" and sys.version_info >= (3, 8):
+        dll_name = "libffi-7.dll"
+        dll_path = os.path.join(sys.base_prefix, "DLLs", dll_name)
+        finder.IncludeFiles(dll_path, os.path.join("lib", dll_name))
+
+
 def load_cx_Oracle(finder, module):
     """the cx_Oracle module implicitly imports datetime; make sure this
        happens."""
     finder.IncludeModule("datetime")
-    try:
-        finder.IncludeModule("decimal")
-    except ImportError:
-        pass
+    finder.IncludeModule("decimal")
 
 
 def load_datetime(finder, module):
@@ -255,17 +264,21 @@ def load_idna(finder, module):
     """the idna module implicitly loads data; make sure this happens."""
     finder.IncludeModule("idna.idnadata")
 
+
 def load_matplotlib(finder, module):
     """the matplotlib module requires data to be found in mpl-data in the
        same directory as the frozen executable so oblige it"""
     import matplotlib
     dataPath = matplotlib.get_data_path()
-    finder.IncludeFiles(dataPath, "mpl-data", copyDependentFiles=False)
+    targetPath = os.path.join("lib", "matplotlib", "mpl-data")
+    finder.AddConstant("MATPLOTLIBDATA", targetPath)
+    finder.IncludeFiles(dataPath, targetPath, copyDependentFiles=False)
 
 
 def load_numpy(finder, module):
     finder.IncludePackage("numpy")
 
+
 def load_matplotlib_numerix(finder, module):
     """the numpy.numerix module loads a number of modules dynamically"""
     for name in ("ma", "fft", "linear_algebra", "random_array", "mlab"):
@@ -418,6 +431,7 @@ def load_pty(finder, module):
     """The sgi module is not needed for this module to function."""
     module.IgnoreName("sgi")
 
+
 def load_pycparser(finder, module):
     """ These files are missing which causes
         permission denied issues on windows when they are regenerated.
@@ -425,6 +439,7 @@ def load_pycparser(finder, module):
     finder.IncludeModule("pycparser.lextab")
     finder.IncludeModule("pycparser.yacctab")
 
+
 def load_pydoc(finder, module):
     """The pydoc module will work without the Tkinter module so ignore the
        error if that module cannot be found."""
@@ -443,6 +458,30 @@ def load_pythoncom(finder, module):
             copyDependentFiles = False)
 
 
+def load_pytz(finder, module):
+    """the pytz module requires timezone data to be found in a known directory
+       or in the zip file where the package is written"""
+    import pytz
+    targetPath = os.path.join("lib", "pytz", "zoneinfo")
+    dataPath = os.path.join(os.path.dirname(pytz.__file__), "zoneinfo")
+    if not os.path.isdir(dataPath):
+        # Fedora (and possibly other systems) use a separate location to
+        # store timezone data so look for that here as well
+        if hasattr(pytz, '_tzinfo_dir'):
+            dataPath = pytz._tzinfo_dir
+        else:
+            dataPath = os.getenv('PYTZ_TZDATADIR') or "/usr/share/zoneinfo"
+        if dataPath.endswith(os.sep):
+            dataPath = dataPath[:-1]
+        if os.path.isdir(dataPath):
+            finder.AddConstant("PYTZ_TZDATADIR", targetPath)
+    if os.path.isdir(dataPath):
+        if module.WillBeStoredInFileSystem():
+            finder.IncludeFiles(dataPath, targetPath, copyDependentFiles=False)
+        else:
+            finder.ZipIncludeFiles(dataPath, "pytz/zoneinfo")
+
+
 def load_pywintypes(finder, module):
     """the pywintypes module is actually contained in a DLL but since those
        cannot be loaded directly in Python 2.5 and higher a special module is
@@ -655,7 +694,6 @@ def load_tkinter(finder, module):
        runtime."""
     if sys.platform == "win32":
         import tkinter
-        import _tkinter
         root_names = "tcl", "tk"
         environ_names = "TCL_LIBRARY", "TK_LIBRARY"
         version_vars = tkinter.TclVersion, tkinter.TkVersion
@@ -666,32 +704,14 @@ def load_tkinter(finder, module):
             except KeyError:
                 lib_texts = os.path.join(sys.base_prefix, "tcl",
                         mod_name + str(ver_var))
-            finder.IncludeFiles(lib_texts, mod_name)
-        for ver_var, mod_name in zip(version_vars, root_names):
+            targetPath = os.path.join("lib", "tkinter", mod_name)
+            finder.AddConstant(env_name, targetPath)
+            finder.IncludeFiles(lib_texts, targetPath)
             dll_name = mod_name + str(ver_var).replace(".", "") + "t.dll"
-            dll_path = os.path.join(sys.base_prefix, "Dlls", dll_name)
+            dll_path = os.path.join(sys.base_prefix, "DLLs", dll_name)
             finder.IncludeFiles(dll_path, os.path.join("lib", dll_name))
 
 
-def load_Tkinter(finder, module):
-    """the Tkinter module has data files that are required to be loaded so
-       ensure that they are copied into the directory that is expected at
-       runtime."""
-    import Tkinter
-    import _tkinter
-    tk = _tkinter.create()
-    tclDir = os.path.dirname(tk.call("info", "library"))
-    # on OS X, Tcl and Tk are organized in frameworks, different layout
-    if sys.platform == 'darwin' and tk.call('tk', 'windowingsystem') == 'aqua':
-        tclSourceDir=os.path.join(os.path.split(tclDir)[0], 'Tcl')
-        tkSourceDir = tclSourceDir.replace('Tcl', 'Tk')
-    else:
-        tclSourceDir = os.path.join(tclDir, "tcl%s" % _tkinter.TCL_VERSION)
-        tkSourceDir = os.path.join(tclDir, "tk%s" % _tkinter.TK_VERSION)
-    finder.IncludeFiles(tclSourceDir, "tcl")
-    finder.IncludeFiles(tkSourceDir, "tk")
-
-
 def load_tempfile(finder, module):
     """the tempfile module attempts to load the fcntl and thread modules but
        continues if these modules cannot be found; ignore these modules if they
@@ -872,4 +892,4 @@ def load_sqlite3(finder, module):
         dll_name = "sqlite3.dll"
         dll_path = os.path.join(sys.base_prefix, "DLLs", dll_name)
         finder.IncludeFiles(dll_path, os.path.join("lib", dll_name))
-
+    finder.IncludePackage('sqlite3')
